<?php

namespace Fastrack\CalendarEvents;

use \DateTime;
use SilverStripe\View\ArrayData;

class CalendarEventsPage_Controller extends \PageController
{
	private static $allowed_actions = [
		'location',
		'calendarEventPage'
	];

	private static $url_handlers = [
		'' => 'index',
		'$Action!/$CalendarEventSlug!' => 'calendarEventPage',
		'$Action!' => 'location'
	];

	const EVENT_NOT_FOUND_MESSAGE = 'Event could not be found';
	const REGION_NOT_FOUND_MESSAGE = 'Region could not be found';

	public function index()
	{
		$this->checkConstants();
		if (!HAS_CALENDAR_EVENTS_PAGE) {
			return $this->httpError(404, 'Page not found');
		}

		$events = CalendarEvent::get()->filter([
				'Active' => true
			])->filterAny([
				'FinishAt:GreaterThanOrEqual' => date_format(new DateTime('now'), 'd/m/Y'),
				'StartAt:GreaterThanOrEqual' => date_format(new DateTime('now'), 'd/m/Y'),
				'RepeatsWeekly' => true,
			])->Sort('StartAt');
		$data = new ArrayData([
			'CalendarEvents' => $events,
			'Region' => false,
			'Categories' => CalendarEventCategory::get()
		]);
		$this->extend('updateCalendarEventsPage', $data);
		return $this->customise($data)->renderWith(['CalendarEvents', 'Page']);
	}

	public function calendarEventPage()
	{
		$event = CalendarEvent::get()->filter([
			'Slug' => $this->getRequest()->param('CalendarEventSlug'),
			'Active' => true
		])
		->filterAny([
			'FinishAt:GreaterThanOrEqual' => date_format(new DateTime('now'), 'd/m/Y'),
			'StartAt:GreaterThanOrEqual' => date_format(new DateTime('now'), 'd/m/Y'),
            'RepeatsWeekly' => true,
		])->first();
		if (!$event) {
			return $this->httpError(404, self::EVENT_NOT_FOUND_MESSAGE);
		}
		if ($event->Region()->Slug !== $this->getRequest()->param('Action')) {
			return $this->httpError(404, self::EVENT_NOT_FOUND_MESSAGE);
		}
		$event->Region = $event->Region();
		$data = $event->getSeoData();
		$data['CalendarEvents'] = CalendarEvents::get()
            ->filter([
                'Slug:not' => $event->Slug,
                'Active' => true,
            ])
            ->filterAny([
                'FinishAt:GreaterThanOrEqual' => date_format(new DateTime('now'), 'd/m/Y'),
                'StartAt:GreaterThanOrEqual' => date_format(new DateTime('now'), 'd/m/Y'),
                'RepeatsWeekly' => true,
            ]);
		$data['Region'] = $event->Region()->Slug;
		$data['CalendarEventCategories'] = CalendarEventCategory::get();
		$data['CalendarEvent'] = $event;
		$arrayData = new ArrayData($data);
		$this->extend('updateCalendarEventPage', $arrayData);
		return $this->customise($arrayData)->renderWith(['CalendarEvent', 'Page']);
	}

	public function location()
	{
		$this->checkConstants();
		if (!HAS_CALENDAR_EVENTS_PAGE) {
			return $this->httpError(404, 'Page not found');
		}
		$template = USE_CALENDAR_EVENTS_REGION_TEMPLATE ? 'CalendarEventsLocation' : 'CalendarEvent';
		$regionName = $this->getRequest()->param('Action');
		$region = Region::get()->filter(['Slug' => $regionName])->first();
		if (!$region) {
			return $this->httpError(404, self::REGION_NOT_FOUND_MESSAGE);
		}
		return $this->customise(new ArrayData([
			'Region' => $region
		]))->renderWith([$template, 'Page']);
	}

	private function checkConstants()
	{
		if (!defined('HAS_CALENDAR_EVENTS_PAGE')) {
			define('HAS_CALENDAR_EVENTS_PAGE', true);
		}
		if (!defined('USE_CALENDAR_EVENTS_REGION_TEMPLATE')) {
			define('USE_CALENDAR_EVENTS_REGION_TEMPLATE', false);
		}
	}
}
